import java.util.Random;

public class Consumer implements Runnable {

  private final EnergySource src;
  private final int id;
  private final Random r = new Random();

  public Consumer(int id, EnergySource src) {
    this.id = id;
    this.src = src;
  }

  private long amountToConsume(long amountRemaining) {
    long input = Math.max(1L, amountRemaining);
    input = (long)(input * 0.25);
    input = Math.max(1L, input);
    return (long)r.nextInt((int)input) + 1;
  }

  public void run() {
    for (int i = 0; i < 10; i++) {
      src.monitor.writeLock().lock();
      try {
        long amountToConsume = amountToConsume(src.getUnitsAvailable());
        System.out.print("Consumer <" + id + ":" + (i+1) + ">");
        System.out.print(" tries to consume");
        System.out.print(" " + amountToConsume + ": ");
        boolean result = src.useEnergy(amountToConsume);
        if (result) {
          System.out.println(" SUCCESS!");
        } else {
          System.out.println(" FAIL!");
        }
      } finally {
        src.monitor.writeLock().unlock();
      }
      try {
        Thread.sleep(r.nextInt(3000));
      } catch (Exception ex) {
      }
    }
  }

}
